from typing import Any, List
import argparse
from pathlib import Path

import pandas as pd
from sklearn.linear_model import LinearRegression


def load_train_data(in_dir: str) -> pd.DataFrame:
    """学習データを読み込む
    :param in_dir: 入力ファイルを格納したディレクトリ
    :return: 学習データ
    """
    return pd.read_csv(Path(in_dir) / "train.csv")


def load_test_data(in_dir: str) -> pd.DataFrame:
    """適用先データを読み込む
    :param in_dir: 入力ファイルを格納したディレクトリ
    :return: 適用対象のデータ
    """
    return pd.read_csv(Path(in_dir) / "test.csv")


def fit_lr(train_df: pd.DataFrame) -> Any:
    """学習済みモデルを作成する
    :param train_df: 学習データを格納したDataFrame
    :return: 学習済みモデル
    """
    # 線形回帰
    lr = LinearRegression()
    X = train_df[["score_a", "score_b"]]
    y = train_df["score_c"]
    lr.fit(X, y)
    return lr


def predict(
    learned_model: Any, test_df: pd.DataFrame
) -> pd.DataFrame:
    """モデルを使って予測する
    :param learned_model: 使用する学習済みモデル
    :param test_df: 適用対象のデータを格納したDataFrame
    :return: 予測結果
    """
    return learned_model.predict(
        test_df[["score_a", "score_b"]]
    )


def output_result(
    out_dir: str,
    test_df: pd.DataFrame,
    result: List[Any],
) -> None:
    """結果を出力する
    :param out_dir: 出力先ディレクトリ
    :param test_df: 適用対象のデータを格納したDataFrame
    :param result: 予測結果
    """
    result_df = pd.DataFrame(
        {"name": test_df["name"], "score_c": result}
    )
    result_df.to_csv(
        Path(out_dir) / "result.csv", index=False
    )
    print(result_df)


def process(args) -> None:
    # データを読み込み
    train_df = load_train_data(args.in_dir)

    # 学習
    lr = fit_lr(train_df)

    # 適用
    test_df = load_test_data(args.in_dir)
    result = predict(lr, test_df)

    # 結果をファイルに出力
    output_result(args.out_dir, test_df, result)


if __name__ == "__main__":
    # コマンドラインパラメータの登録
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-i",
        "--in_dir",
        type=str,
        help="入力ファイルを格納するディレクトリ",
    )
    parser.add_argument(
        "-o",
        "--out_dir",
        type=str,
        help="出力ファイルを格納するディレクトリ",
    )
    args = parser.parse_args()
    process(args)
